#!/usr/bin/env perl
# StarDict SYN to standalone dict converter
# © 2011, Deep IT, TranslateIt! Project.
# http://mac.gettranslateit.com/

use strict;

my $force = 0;
$force = 1 if (scalar(@ARGV) && $ARGV[0] eq '--force');

sub readFile
{
	my $name = shift;

	local $/;
	undef $/;

	my $content = '';
	local * F;
	my $ok = 1;
	open(F,"<$name") or $ok = 0;
	if ($ok)
	{
		binmode(F);
		$content = <F>;
		close(F);
	}

	return $content;
}

sub processSYN
{
	my $synName = shift;

	if (-f $synName)
	{
		my $dictName = $synName;
		$dictName =~ s/\.syn$//soi;
		my $idxName = $dictName . '.idx';
		if (-f $idxName)
		{
			my $resDict = $dictName . '-syn.dict';
			if (! -f $resDict || $force)
			{
				my $resIDX = $dictName . '-syn.idx';
				if (! -f $resIDX || $force)
				{
					print "Processing $dictName...\n";
					my $syn = readFile($synName);
					if (length($syn))
					{
						my $idx = readFile($idxName);
						if (length($idx))
						{
							# parsing IDX to dereference SYN records
							my @idxwords;
							while($idx =~ /([^\x00]+)\x00.{4}.{4}/gos)
							{
								push(@idxwords,$1);
							}
							# walking thr. syns
							my @syns;
							while ($syn =~ /([^\x00]+)\x00(.{4})/gos)
							{
								my $syn = $1;
								my $sidx = unpack('N',$2);
								my $refWord = $idxwords[$sidx];
								if ($refWord)
								{
									push(@syns,{'w' => $syn,'r' => $refWord});
								}
							}
							local * OD;
							my $ok = 1;
							open(OD,">$resDict") or $ok = 0;
							if ($ok)
							{
								local * OI;
								open(OI,">$resIDX") or $ok = 0;
								if ($ok)
								{
									foreach my $word (@syns)
									{
										my $offset = tell(OD);
										my $entry = '{' . $word->{'r'} . '}';
										print OD $entry;
										print OI $word->{'w'} . "\x00";
										print OI pack('N',$offset);
										print OI pack('N',length($entry));
									}
									close(OI);
								}
								close(OD);
							}
						}
						else
						{
							print " Failed: IDX is empty\n";
						}
					}
					else
					{
						print " Failed: SYN is empty\n";
					}
				}
				else
				{
					print " '$resIDX' is already exist\n";
				}
			}
			else
			{
				print " '$resDict' is already exist\n";
			}
		}
		else
		{
			print "Failed: no '$idxName' file found\n";
		}
	}
	else
	{
		print "Failed: no '$synName' file found\n";
	}
}

local * DIR;
opendir(DIR,'.');
my $item;
while (($item = readdir(DIR)))
{
	next if ($item =~ /^\.{1,2}$/so);
	next unless ($item =~ /\.syn$/soi);
	processSYN($item);
}
closedir(DIR);
